@@ -94,6 +94,7 @@ from hypothesis.searchstrategy.datetime import (
     TimedeltaStrategy,
 )
 from hypothesis.searchstrategy.deferred import DeferredStrategy
+from hypothesis.searchstrategy.functions import FunctionStrategy
 from hypothesis.searchstrategy.lazy import LazyStrategy
 from hypothesis.searchstrategy.misc import (
     BoolStrategy,
@@ -717,7 +718,7 @@ def lists(
     elements,  # type: SearchStrategy[Ex]
     min_size=0,  # type: int
     max_size=None,  # type: int
-    unique_by=None,  # type: Callable[..., Any]
+    unique_by=None,  # type: Union[Callable, Tuple[Callable, ...]]
     unique=False,  # type: bool
 ):
     # type: (...) -> SearchStrategy[List[Ex]]
@@ -730,10 +731,20 @@ def lists(
     object equality, as if unique_by was ``lambda x: x``. This comparison only
     works for hashable types.
 
-    if unique_by is not None it must be a function returning a hashable type
-    when given a value drawn from elements. The resulting list will satisfy the
-    condition that for ``i`` != ``j``, ``unique_by(result[i])`` !=
-    ``unique_by(result[j])``.
+    If unique_by is not None it must be a callable or tuple of callables
+    returning a hashable type when given a value drawn from elements. The
+    resulting list will satisfy the condition that for ``i`` != ``j``,
+    ``unique_by(result[i])`` != ``unique_by(result[j])``.
+
+    If ``unique_by`` is a tuple of callables the uniqueness will be respective
+    to each callable.
+
+    For example, the following will produce two columns of integers with both
+    columns being unique respectively.
+    .. code-block:: pycon
+
+        >>> twoints = st.tuples(st.integers(), st.integers())
+        >>> st.lists(twoints, unique_by=(lambda x: x[0], lambda x: x[1]))
 
     Examples from this strategy shrink by trying to remove elements from the
     list, and by shrinking each individual element of the list.
@@ -754,8 +765,19 @@ def lists(
     if max_size == 0:
         return builds(list)
     if unique_by is not None:
+        if not (callable(unique_by) or isinstance(unique_by, tuple)):
+            raise InvalidArgument(
+                "unique_by=%r is not a callable or tuple of callables" % (unique_by)
+            )
+        if callable(unique_by):
+            unique_by = (unique_by,)
+        if len(unique_by) == 0:
+            raise InvalidArgument("unique_by is empty")
+        for i, f in enumerate(unique_by):
+            if not callable(f):
+                raise InvalidArgument("unique_by[%i]=%r is not a callable" % (i, f))
         return UniqueListStrategy(
-            elements=elements, max_size=max_size, min_size=min_size, key=unique_by
+            elements=elements, max_size=max_size, min_size=min_size, keys=unique_by
         )
     return ListStrategy(elements, min_size=min_size, max_size=max_size)
 
@@ -1214,7 +1236,7 @@ def builds(
             "builds(), but is only allowed as a keyword arg"
         )
     required = required_args(target, args, kwargs) or set()
-    to_infer = set(k for k, v in kwargs.items() if v is infer)
+    to_infer = {k for k, v in kwargs.items() if v is infer}
     if required or to_infer:
         if isclass(target) and attr.has(target):
             # Use our custom introspection for attrs classes
@@ -1283,6 +1305,25 @@ def from_type(thing):
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
        it is resolved via :func:`~hypothesis.strategies.builds`.
+
+    There is a valuable recipe for leveraging ``from_type()`` to generate
+    "everything except" values from a specified type. I.e.
+
+    .. code-block:: python
+
+        def everything_except(excluded_types):
+            return (
+                from_type(type).flatmap(from_type)
+                .filter(lambda x: not isinstance(x, excluded_types))
+            )
+
+    For example, ``everything_except(int)`` returns a strategy that can
+    generate anything that ``from_type()`` can ever generate, except for
+    instances of :class:python:int, and excluding instances of types
+    added via :func:~hypothesis.strategies.register_type_strategy.
+
+    This is useful when writing tests which check that invalid input is
+    rejected in a certain way.
     """
     # TODO: We would like to move this to the top level, but pending some major
     # refactoring it's hard to do without creating circular imports.
@@ -2030,6 +2071,13 @@ def runner(default=not_set):
 
 
 class DataObject(object):
+    """This type only exists so that you can write type hints for tests using
+    the :func:`~hypothesis.strategies.data` strategy.  Do not use it directly!
+    """
+
+    # Note that "only exists" here really means "is only exported to users",
+    # but we want to treat it as "semi-stable", not document it as "public API".
+
     def __init__(self, data):
         self.count = 0
         self.conjecture_data = data
@@ -2038,6 +2086,7 @@ class DataObject(object):
         return "data(...)"
 
     def draw(self, strategy, label=None):
+        # type: (SearchStrategy[Ex], Any) -> Ex
         result = self.conjecture_data.draw(strategy)
         self.count += 1
         if label is not None:
@@ -2074,17 +2123,14 @@ class DataStrategy(SearchStrategy):
 
     def __not_a_first_class_strategy(self, name):
         raise InvalidArgument(
-            (
-                "Cannot call %s on a DataStrategy. You should probably be "
-                "using @composite for whatever it is you're trying to do."
-            )
-            % (name,)
+            "Cannot call %s on a DataStrategy. You should probably be using "
+            "@composite for whatever it is you're trying to do." % (name,)
         )
 
 
 @cacheable
 def data():
-    # type: () -> SearchStrategy[Any]
+    # type: () -> SearchStrategy[DataObject]
     """This isn't really a normal strategy, but instead gives you an object
     which can be used to draw data interactively from other strategies.
 
@@ -2189,3 +2235,72 @@ def emails():
     return builds(u"{}@{}".format, local_part, domains()).filter(
         lambda addr: len(addr) <= 254
     )
+
+
+# Mypy can't yet handle default values with generic types or typevars, but the
+# @overload workaround from https://github.com/python/mypy/issues/3737 doesn't
+# work with @composite functions - Mypy can't see that the function implements
+# `(Any, Callable[..., T], SearchStrategy[T]) -> Callable[..., T]`
+
+
+@defines_strategy
+def functions(like=lambda: None, returns=none()):
+    """A strategy for functions, which can be used in callbacks.
+
+    The generated functions will mimic the interface of ``like``, which must
+    be a callable (including a class, method, or function).  The return value
+    for the function is drawn from the ``returns`` argument, which must be a
+    strategy.
+
+    Note that the generated functions do not validate their arguments, and
+    may return a different value if called again with the same arguments.
+
+    Generated functions can only be called within the scope of the ``@given``
+    which created them.  This strategy does not support ``.example()``.
+    """
+    check_type(SearchStrategy, returns)
+    if not callable(like):
+        raise InvalidArgument(
+            "The first argument to functions() must be a callable to imitate, "
+            "but got non-callable like=%r" % (nicerepr(like),)
+        )
+    return FunctionStrategy(like, returns)
+
+
+@composite
+def slices(draw, size):
+    """Generates slices that will select indices up to the supplied size
+
+    Generated slices will have start and stop indices that range from 0 to size - 1
+    and will step in the appropriate direction. Slices should only produce an empty selection
+    if the start and end are the same.
+
+    Examples from this strategy shrink toward 0 and smaller values
+    """
+    check_valid_integer(size)
+    if size is None or size < 1:
+        raise InvalidArgument("size=%r must be at least one" % size)
+
+    min_start = min_stop = 0
+    max_start = max_stop = size
+    min_step = 1
+    # For slices start is inclusive and stop is exclusive
+    start = draw(integers(min_start, max_start) | none())
+    stop = draw(integers(min_stop, max_stop) | none())
+
+    # Limit step size to be reasonable
+    if start is None and stop is None:
+        max_step = size
+    elif start is None:
+        max_step = stop
+    elif stop is None:
+        max_step = start
+    else:
+        max_step = abs(start - stop)
+
+    step = draw(integers(min_step, max_step or 1))
+
+    if (stop or 0) < (start or 0):
+        step *= -1
+
+    return slice(start, stop, step)
